//=======================================================================
// Author: Donovan Parks
//
// Copyright 2009 Donovan Parks
//
// This file is part of GenGIS.
//
// GenGIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// GenGIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GenGIS.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#ifndef _GENGIS_LAYER_TREE_VIEW_
#define _GENGIS_LAYER_TREE_VIEW_

#include "../core/Precompiled.hpp"

namespace GenGIS 
{

/**
 * @brief View of tree structure which contains all layers.
 */
class LayerTreeView
{
public:
	//** Possible image states for the layer tree view. */
	enum IMAGE_STATE { UNCHECKED = 0, CHECKED = 1 };

public:
	/** Constructor. */
	LayerTreeView(LayerTreePtr tree, wxTreeCtrl* treeCtrl);

	/** Destructor. */
	~LayerTreeView() {}

	/** Get layer tree control. */
	wxTreeCtrl* GetTreeCtrl() { return m_treeCtrl; }

	/** 
	 * @breif Set checked/unchecked state of tree item. 
	 * @param treeId Id of tree item to change state of.
	 * @param state State to set tree item to.
	 */
	void SetChecked(const wxTreeItemId& treeId, bool bVisible);

	/** 
	 * @breif Set label name of tree item. 
	 * @param treeId Id of tree item to change name of
	 * @param name New name of tree item.
	 */
	void SetName(const wxTreeItemId& treeId, const std::wstring& name);

protected:
	/**
	 * @brief Traverse all children and set their text colour.
	 * @param id Id of top node to traverse from.
	 * @param cookie Required to make function reentrant (see wxTreeCtrl documentation).
	 * @param stopAtUnchecked Flag indicating if traversal should stop once an unchecked child is encountered.
	 * @param colour Colour to set text to.
	 */
	void SetColourAllChildren(wxTreeItemId id, wxTreeItemIdValue cookie, bool stopAtUnchecked, wxColour colour);

protected:
	/** Layer tree model. */
	LayerTreePtr m_tree;

	/** View of layer tree model. */
	wxTreeCtrl* m_treeCtrl;
};

}

#endif
