//=======================================================================
// Author: Donovan Parks
//
// Copyright 2009 Donovan Parks
//
// This file is part of GenGIS.
//
// GenGIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// GenGIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GenGIS.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#include "../core/Precompiled.hpp"

#include "../core/LocationLayer.hpp"
#include "../core/SequenceLayer.hpp"
#include "../core/LocationController.hpp"

using namespace GenGIS;

void LocationLayer::RemoveSequence(unsigned int index)
{
	m_sequence.erase(m_sequence.begin() + index);
}

LayerPtr LocationLayer::GetLayer(uint id) const
{
	for(unsigned int index = 0; index < GetNumSequences(); index++)
	{
		SequenceLayerPtr layer = GetSequence(index);
		if(layer->GetId() == id)
			return layer;

		LayerPtr returnLayer = layer->GetLayer(id);
		if(returnLayer != LayerPtr())
		{
			return returnLayer;
		}
	}

	return LayerPtr();
}


bool LocationLayer::RemoveLayer(uint id)
{
	bool bFound = false;
	for(unsigned int index = 0; index < GetNumSequences() && !bFound; index++)
	{
		SequenceLayerPtr layer = GetSequence(index);
		if(layer->GetId() == id)
		{
			RemoveSequence(index);
			return true;
		}

		bFound = layer->RemoveLayer(id);
	}

	return bFound;
}

void LocationLayer::Render()
{
	if(!IsVisible())
		return;

	m_locationController->Render();

	for(unsigned int index = 0; index < GetNumSequences(); index++)
	{
		GetSequence(index)->Render();
	}
}
