//=======================================================================
// Author: Donovan Parks
//
// Copyright 2009 Donovan Parks
//
// This file is part of GenGIS.
//
// GenGIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// GenGIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GenGIS.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#ifndef _GENGIS_SEQUENCE_CONTROLLER
#define _GENGIS_SEQUENCE_CONTROLLER

#include "../core/Precompiled.hpp"

#include "../core/SequenceModel.hpp"

namespace GenGIS
{

/**
 * @brief Handle events related to a given sequence.
 */
class SequenceController
{
public:
	/** 
	 * @brief Constuctor.
	 * @param sequenceModel Sequence model to associate with this controller.
	 */
	SequenceController(SequenceModelPtr sequenceModel): m_sequenceModel(sequenceModel) {}

	/** Destructor. */
	virtual ~SequenceController() {}

	/** Get id of sequence. */
	const std::wstring GetSequenceId() { return m_sequenceModel->GetSequenceId(); }

	/** Get id of location site associated with sequence. */
	const std::wstring GetSiteId() { return m_sequenceModel->GetSiteId(); }

	/** Get flag indicating if sequence is in active set. */
	bool IsActive() { return m_sequenceModel->IsActive(); }

	/** Set flag indicating if sequence is in active set. */
	void SetActive(bool state) { m_sequenceModel->SetActive(state); }

	/** Get data associate with sequence. */
	std::map<std::wstring,std::wstring> GetData() { return m_sequenceModel->GetData(); }

	/** 
	 * @brief Get data for specified field. 
	 * @param field Field to get data for. 
	 * @return Data associated with the provided field.
	 */
	const std::wstring& GetData(const std::wstring& field) { return m_sequenceModel->GetData(field); }

	/** Set field being used to select active sequences. */
	static void SetActiveField(const std::wstring& activeField) { m_activeField = activeField; }

	/** Get field being used to select active sequences. */
	static const std::wstring& GetActiveField() { return m_activeField; }

protected:
	/** Location model associated with this controller. */
	SequenceModelPtr m_sequenceModel;
	
	/** Field being used to select active sequences. */
	static std::wstring m_activeField; 
};

}

#endif
