//=======================================================================
// Author: Donovan Parks
//
// Copyright 2009 Donovan Parks
//
// This file is part of GenGIS.
//
// GenGIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// GenGIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GenGIS.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#ifndef _GENGIS_SEQUENCE_LAYER_
#define _GENGIS_SEQUENCE_LAYER_

#include "../core/Precompiled.hpp"

#include "../core/Layer.hpp"

namespace GenGIS
{

/**
 * @brief Sequence data collected from a location.
 */
class SequenceLayer : public Layer
{
public:
	/** Convenient syntax for creating smart pointers of this class. */
	typedef boost::shared_ptr<SequenceLayer> Ptr;

public:
	/** 
	 * @brief Constuctor.
	 * @param id Unique identifier used to specify sample.
	 * @param parent Parent of this layer.
	 * @param sequenceController Sequence controller to associate with this layer.
	 */
	SequenceLayer(uint id, LayerPtr parent, SequenceControllerPtr sequenceController)
		: Layer(id, Layer::SEQUENCE, parent), m_sequenceController(sequenceController) {}

	/** Destructor. */
	virtual ~SequenceLayer() {}

	/**
	 * @brief Get layer from tree below this layer. 
	 * @param id Unique id of desired layer.
	 * @return layer at given id.
	 */
	virtual LayerPtr GetLayer(uint id) const;

	/**
	 * @brief Remove layer from tree below this layer. 
	 * @param id Unique id of layer to remove.
	 * @return True if layer removed, false if layer not found.
	 */
	virtual bool RemoveLayer(uint id);

	/** Render contents of layer. */
	virtual void Render();

	/** Get location controller associated with this layer. */
	SequenceControllerPtr GetSequenceController() { return m_sequenceController; }
        
private:
	SequenceControllerPtr m_sequenceController;
};

}


#endif
