//=======================================================================
// Author: Donovan Parks
//
// Copyright 2009 Donovan Parks
//
// This file is part of GenGIS.
//
// GenGIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// GenGIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GenGIS.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#ifndef _GENGIS_STUDY_CONTROLLER_
#define _GENGIS_STUDY_CONTROLLER_

#include "../core/Precompiled.hpp"

#include "../core/StudyModel.hpp"

namespace GenGIS
{

/**
 * @brief Handle events related to a given study.
 */
class StudyController
{
public:
	/** Construction. */
	StudyController(StudyModelPtr studyModel): m_studyModel(studyModel) {}

	/** Destructor. */
	~StudyController() {}

	/**
	 * @brief Get projection tool.
	 * @param oSource Coordinate system of source. If NULL, assume lat/lon with same datum as projected coordinate system.
	 * @return Projection tool.
	 */
	ProjectionToolPtr GetProjectionTool(OGRSpatialReference* oSource = NULL)	{	return m_studyModel->GetProjectionTool(oSource);	}

	/** Get specified destination coordinate system. */
	OGRSpatialReference GetDestinationProj() { return m_studyModel->GetDestinationProj(); }

	/** Set flag indicating if data should be projected. */
	void SetProject(bool state) { m_studyModel->SetProject(state); }

	/** Get flag indicating if data should be projected. */
	bool GetProject() { return m_studyModel->GetProject(); }

	/** Set datum. */
	void SetDatum(const std::wstring& datum) { m_studyModel->SetDatum(datum); }

	/** Get datum. */
	const std::wstring& GetDatum() const { return m_studyModel->GetDatum(); }

	/** Set projection. */
	void SetProjection(const std::wstring& projection) { m_studyModel->SetProjection(projection); }

	/** Get projection. */
	const std::wstring& GetProjection() const { return m_studyModel->GetProjection(); }

	/** Set centre latitude. */
	void SetCentreLatitude(float centre) { m_studyModel->SetCentreLatitude(centre); }

	/** Get centre Longitude. */
	float GetCentreLongitude() { return m_studyModel->GetCentreLongitude(); }

	/** Set centre Longitude. */
	void SetCentreLongitude(float centre)  { m_studyModel->SetCentreLongitude(centre); }

	/** Get centre latitude. */
	float GetCentreLatitude() { return m_studyModel->GetCentreLatitude(); }

	/** Set first standard parallel. */
	void SetFirstStandardParallel(float sp) { m_studyModel->SetFirstStandardParallel(sp); }

	/** Get first standard parallel. */
	float GetFirstStandardParallel() { return m_studyModel->GetFirstStandardParallel(); }

	/** Set second standard parallel. */
	void SetSecondStandardParallel(float sp) { m_studyModel->SetSecondStandardParallel(sp); }

	/** Get second standard parallel. */
	float GetSecondStandardParallel() { return m_studyModel->GetSecondStandardParallel(); }

protected:
	/** Study model associated with this controller. */
	StudyModelPtr m_studyModel;
};

}

#endif
