//=======================================================================
// Author: Donovan Parks
//
// Copyright 2009 Donovan Parks
//
// This file is part of GenGIS.
//
// GenGIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// GenGIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GenGIS.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#include "../core/Precompiled.hpp"

#include "../core/StudyLayer.hpp"
#include "../core/MapLayer.hpp"
#include "../core/TerrainController.hpp"
#include "../core/TerrainView.hpp"

using namespace GenGIS;

MapLayerPtr StudyLayer::GetMap(const std::wstring& name) const
{
	std::vector<MapLayerPtr>::const_iterator iter;
	for(iter = m_maps.begin(); iter != m_maps.end(); iter++)
	{
		if((*iter)->GetName() == name)
		{
			return *iter;
		}
	}

	return MapLayerPtr();
}

void StudyLayer::RemoveMap(unsigned int index)
{
	// erase study and all children 
	for(unsigned int i = 0; i < m_maps.at(index)->GetNumLocationSets(); ++i)
	{
		m_maps.at(index)->RemoveLocationSet(0);	// keep removing the first element
	}

	m_maps.erase(m_maps.begin() + index);
}

void StudyLayer::RemoveMap(const std::wstring& name)
{
	std::vector<MapLayerPtr>::iterator iter;
	for(iter = m_maps.begin(); iter != m_maps.end(); iter++)
	{
		if((*iter)->GetName() == name)
		{
			// erase study and all children 
			for(unsigned int i = 0; i < (*iter)->GetNumLocationSets(); ++i)
			{
				(*iter)->RemoveLocationSet(0);	// keep removing the first element
			}

			m_maps.erase(iter);
			break;
		}
	}
}

LayerPtr StudyLayer::GetLayer(uint id) const
{
	for(unsigned int index = 0; index < GetNumMaps(); index++)
	{
		MapLayerPtr layer = GetMap(index);
		if(layer->GetId() == id)
			return layer;

		LayerPtr returnLayer = layer->GetLayer(id);
		if(returnLayer != LayerPtr())
		{
			return returnLayer;
		}
	}

	return LayerPtr();
}

bool StudyLayer::RemoveLayer(uint id)
{
	bool bFound = false;
	for(unsigned int index = 0; index < GetNumMaps() && !bFound; index++)
	{
		MapLayerPtr layer = GetMap(index);
		if(layer->GetId() == id)
		{
			RemoveMap(index);
			return true;
		}

		bFound = layer->RemoveLayer(id);
	}

	return bFound;
}

void StudyLayer::Render()
{
	if(!IsVisible())
		return;

	for(unsigned int index = 0; index < GetNumMaps(); index++)
	{
		GetMap(index)->Render();
	}
}
