//=======================================================================
// Author: Donovan Parks
//
// Copyright 2009 Donovan Parks
//
// This file is part of GenGIS.
//
// GenGIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// GenGIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GenGIS.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#ifndef _GENGIS_TREE_LAYER_
#define _GENGIS_TREE_LAYER_

#include "../core/Precompiled.hpp"

#include "../core/Layer.hpp"

namespace GenGIS
{

/**
 * @brief A tree layer displays geographical tree models. It is a terminal node in the layer tree model.
 */
class TreeLayer : public Layer
{
public:
	/** Allows convenient syntax for creating smart pointers of this class. */
	typedef boost::shared_ptr<TreeLayer> Ptr;	

public:
	/** 
	 * @brief Constuctor.
	 * @param id Unique identifier used to specify tree layer.
	 * @param parent Parent of this layer.
	 * @param treeController Tree controller associated with this tree layer.
	 */
	TreeLayer(uint id, LayerPtr parent, GeoTreeControllerPtr treeController)
		: Layer(id, Layer::TREE, parent), m_treeController(treeController) {}

	/** Destructor. */
	virtual ~TreeLayer() {}

	/**
	 * @brief Get layer from tree below this layer. 
	 * @param id Unique id of desired layer.
	 * @return layer at given id.
	 */
	virtual LayerPtr GetLayer(uint id) const;

	/**
	 * @brief Remove layer from tree below this layer. 
	 * @param id Unique id of layer to remove.
	 * @return True if layer removed, false if layer not found.
	 */
	virtual bool RemoveLayer(uint id);

	/** Render contents of layer. */
	virtual void Render();

	/** Get geographic tree controller associated with this layer. */
	GeoTreeControllerPtr GetGeoTreeController() { return m_treeController; }

protected:
	/** Controller for geographic tree model. */
	GeoTreeControllerPtr m_treeController;
};

}

#endif
