//=======================================================================
// Author: Donovan Parks
//
// Copyright 2009 Donovan Parks
//
// This file is part of GenGIS.
//
// GenGIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// GenGIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GenGIS.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#ifndef _GENGIS_COLOUR_MAP_WIDGET_
#define _GENGIS_COLOUR_MAP_WIDGET_

#include "../core/Precompiled.hpp"


namespace GenGIS
{

/**
 * @brief Used to track which colour picker trigger an event.
 */
class ColourPickerTracker : public wxObject
{
	public:
		ColourPickerTracker(int index): m_index(index) {}

		int GetIndex() { return m_index; }

	public:
		int m_index;
};

/**
 * @brief Helper functions for interacting with colour maps.
 */
class ColourMapWidget : public wxDialog 
{
public:
	/** Constructor. */
	ColourMapWidget(wxComboBox* cboColourMap): m_cboColourMap(cboColourMap) {}

	/** Destructor */
	~ColourMapWidget() {}

	/** Set colour map. */
	void SetColourMap(ColourMapDiscretePtr colourMap) { m_colourMap = colourMap; }

	/** Get colour map. */
	ColourMapDiscretePtr GetColourMap() const { return m_colourMap; }

	/** Get colour pickers. */
	std::vector<wxColourPickerCtrl*>& GetColourPicker() { return m_colourPickers; }

	/** Get field values. */
	std::vector<wxStaticText*>& GetFieldValues() { return m_fieldValues; }

	/** Populate the wxScrolledWindow with all possible field values. */
	void SetFieldValues(wxScrolledWindow* scrolledWindow, const std::vector<std::wstring>& fieldValues);

	/** Assign colours to field values based on colour map. */
	void SetColourMap();

	/** Populate combo box with all available colour maps. */
	void PopulateColourMapComboBox();

protected:
	/** Specify event for all wxColourPickers. */
	void ConnectColourPickerEvents();

	/** Disconnect event for all wxColourPickers. */
	void DisconnectColourPickerEvents();

	/** Event indicating the colour of a field value has been changed. */
	void OnColourChange( wxColourPickerEvent& event );

private:
	/** Colour map of interest. */
	ColourMapDiscretePtr m_colourMap;

	/** Colour pickers used to display colour map. */
	std::vector<wxColourPickerCtrl*> m_colourPickers;

	/** Labels indicating name associated with a colour. */
	std::vector<wxStaticText*> m_fieldValues;

	/** Tracker used to determine which colour picker triggered an event. */
	std::vector<ColourPickerTracker*> m_trackers;

	/** Combo box to populate with available colour maps. */
	wxComboBox* m_cboColourMap;
};

}

#endif 
