//=======================================================================
// Author: Donovan Parks
//
// Copyright 2009 Donovan Parks
//
// This file is part of GenGIS.
//
// GenGIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// GenGIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GenGIS.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#ifndef _GENGIS_GEOPHYLOGENY_FRAME_
#define _GENGIS_GEOPHYLOGENY_FRAME_

#include "../core/Precompiled.hpp"

#include "../core/GeoTreeView.hpp"

#include "../gui/GenGisLayout.h"

namespace GenGIS
{

/**
 * @brief Handles all events and GUI processing for PluginLayout form.
 */
class GenGisFrame : public GenGisLayout
{
  friend struct APIv1;
public:

public:
	/**
	 * @brief Constructor.
	 * @param parent Parent for this frame
	 */
	GenGisFrame( wxWindow* parent = (wxFrame *)NULL );

	/** Destructor */
	~GenGisFrame();
  
  void AddConsolePanel(wxPanel* panel, wxString name);
  wxNotebook* GetBottomNotebook();
  
  void InitializeTerrainController();
  
  void FillSequenceLegend();
  
  void FillSamplesLegend();
  void OnSamplesLegendPaint(wxPaintEvent& WXUNUSED(event));

	void SafeYield() { ::wxSafeYield(); }

	TerrainControllerPtr GetTerrainController() { return m_terrainController; }
  
private:
  
	/** Connect events handled by this frame. */
	virtual void ConnectEvents();

	/** Disconnect events handled by this frame. */
	virtual void DisconnectEvents();
  
	void OnFileNewStudy( wxCommandEvent& event ) { wxMessageBox(wxT("Not yet implemented."), wxT("Not yet implemented"), wxOK | wxICON_INFORMATION); }
	void OnFileOpenStudy( wxCommandEvent& event ) { wxMessageBox(wxT("Not yet implemented."), wxT("Not yet implemented"), wxOK | wxICON_INFORMATION); }
	void OnFileSaveStudy( wxCommandEvent& event ) { wxMessageBox(wxT("Not yet implemented."), wxT("Not yet implemented"), wxOK | wxICON_INFORMATION); }
	void OnFileSaveStudyAs( wxCommandEvent& event ) { wxMessageBox(wxT("Not yet implemented."), wxT("Not yet implemented"), wxOK | wxICON_INFORMATION); }

	void OnConfigureDB( wxCommandEvent& event );
	void OnObtainDataFromDB( wxCommandEvent& event );
	
	/** File->Save image as event handler. */
	void OnFileSaveImageAs( wxCommandEvent& event );

	/** File->Exit event handler. */
	void OnFileExit( wxCommandEvent& event );

	/** Layer->Open map layer event handler. */
	void OnLayerOpenMap( wxCommandEvent& event );

	/** Open map from command line. */
	void LayerOpenMap(std::string file);

	/** Handle actual loading of map from file. */
  void LayerOpenMap(std::wstring filename, std::wstring path);

	/** Layer->Open location layer event handler. */
	void OnLayerOpenLocations( wxCommandEvent& event );

	/** Open location set from command line. */
	void LayerOpenLocations(std::string file);

	/** Handle actual loading of location set data from file. */
  void LayerOpenLocations(std::wstring filename, std::wstring path);

	/** Layer->Open tree layer event handler. */
	void OnLayerOpenTree( wxCommandEvent& event );

	/** Open tree from command line. */
	void LayerOpenTree(std::string file);

	/** Handle actual loading of tree data from file. */
  void LayerOpenTree(std::wstring filename, std::wstring path);

	/** Layer->Open sequence data event handler. */
	void OnLayerOpenSequenceData( wxCommandEvent& event );

	/** Open sequence data from command line. */
	void LayerOpenSequenceData(std::string file);

	/** Handle actual loading of sequence data from file. */
  void LayerOpenSequenceData(std::wstring filename, std::wstring path);

	/** Layer->Remove event handler. */
	void OnLayerRemove( wxCommandEvent& event );

	/** Layer->Hide All event handler. */
	void OnLayerHideAll( wxCommandEvent& event );

	/** Layer->Show All handler. */
	void OnLayerShowAll( wxCommandEvent& event );

	/** Handle the layer tree select event. */
	void OnLayerTreeSelect( wxTreeEvent& event );

	/** Handle the layer tree left-click event. */
	void OnLayerTreeClick( wxMouseEvent& event );

	/** Handle the layer tree left float click event. */
	void OnLayerTreeLeftDClick( wxMouseEvent& event );

	/** Handle the layer tree right-click event. */
	void OnLayerTreePopupMenu( wxTreeEvent& event );

	/** View->Camera Position->Reset event handler. */
	void OnViewReset( wxCommandEvent& event );
  void ViewReset();
  
	/** View->Camera Position->Top event handler. */
	void OnViewTop( wxCommandEvent& event );
  void ViewTop();

	/** View->Panes->Side Panel event handler. */
	void OnViewSidePanel( wxCommandEvent& event );
	void ViewSidePanel();

	/** View->Panes->Console event handler. */
	void OnViewConsole( wxCommandEvent& event );
  void ViewConsole();

	/** View->Widgets->Compass event handler. */
	void OnViewCompass( wxCommandEvent& event );

	/** View->Widgets->Overview Map event handler. */
	void OnViewMiniMap( wxCommandEvent& event );

	/** View->Widgets->Navigation Control event handler. */
	void OnViewNav( wxCommandEvent& event );

	/** Hide all widgets. */
	void OnViewHideWidgets( wxCommandEvent& event );

	/** Show all widgets. */
	void OnViewShowWidgets( wxCommandEvent& event );

	/** View->Detail->Fine event handler. */
	void OnViewFine( wxCommandEvent& event );
  void ViewFine();

	/** View->Detail->Coarse event handler. */
	void OnViewCoarse( wxCommandEvent& event );
  void ViewCoarse();

	/** View->Select sequence dialog event handler. */
	void OnViewSelectSeqDlg( wxCommandEvent& event );
	void ViewSelectSegDlg();

	/** Settings->Font event handler. */
	void OnFontSettingsDlg( wxCommandEvent& event );

	/** Settings->Lighting event handler. */
	void OnLightingSettingsDlg( wxCommandEvent& event );

	/** Settings->Layout objects event handler. */
	void OnLayoutObjectSettingsDlg( wxCommandEvent& event );

	/** Help->About event handler. */
	void OnHelpAbout( wxCommandEvent& event );

	/** Standard mode toolbar button event handler. */
	void OnModeStandard( wxCommandEvent& event );

	/** Draw layout line toolbar button event handler. */
	void OnModeDrawLayoutLine( wxCommandEvent& event );

	/** Draw layout ellipse toolbar button event handler. */
	void OnModeDrawLayoutEllipse( wxCommandEvent& event );

	/** Label tool toolbar button event handler */
	void OnLabelTool( wxCommandEvent& event );

	/** Define geographic axis using a polyline event handler. */
	void OnGeoAxisByPolyline( wxCommandEvent& event );

	/** Erase layout line popup menu event. */
	void OnLayoutLineDelete(wxCommandEvent& event);

	/** Erase layout ellipse popup menu event. */
	void OnLayoutEllipseDelete(wxCommandEvent& event);

	/** Erase geographic axis polyline popup menu event. */
	void OnGeoAxisPolylineDelete(wxCommandEvent& event);

	/** OpenGL viewport hide popup menu event. */
	void OnSelectedViewHide(wxCommandEvent& event);

	/** OpenGL viewport properties popup menu event. */
	void OnSelectedViewProperties(wxCommandEvent& event);

	/** Remove selected view from viewport popup menu event. */
	void OnSelectedViewRemove(wxCommandEvent& event);

	/** Handle the erase event for the main form which is generated
	 		whenever a window's background needs to be repainted. */
	void OnEraseBackground( wxEraseEvent& event ){ /* prevent flicker */ }

	/** Handle the erase event for the viewport panel which is generated
			whenever a window's background needs to be repainted. */
	void OnViewportEraseBackground( wxEraseEvent& event ){ /* prevent flicker */ }

	/** Handle the study properties popup menu event. */
	void OnStudyProperties(wxCommandEvent& event);

	/** Handle the map properties popup menu event. */
	void OnMapProperties(wxCommandEvent& event);

	/** Handle the location set properties popup menu event. */
	void OnLocationSetProperties(wxCommandEvent& event);

	/** Handle the location properties popup menu event. */
	void OnLocationProperties(wxCommandEvent& event);

	/** Handle the sequence properties popup menu event. */
	void OnSequenceProperties(wxCommandEvent& event);

	/** Handle the tree properties popup menu event. */
	void OnTreeProperties(wxCommandEvent& event);

		/** Handle the popup menu 3D cladogram event. */
	void OnCladogram3D( wxCommandEvent& event );

	/** Handle the popup menu 3D slanted cladogram event. */
	void OnSlantedCladogram3D( wxCommandEvent& event );

	/** Handle the popup menu 2D cladogram event. */
	void OnCladogram2D( wxCommandEvent& event );

	/** Handle the popup menu 2D phylogram event. */
	void OnPhylogram2D( wxCommandEvent& event );

	/** Handle the popup menu 2D slanted cladogram event. */
	void OnSlantedCladogram2D( wxCommandEvent& event );

	/** Handle the popup menu ellipse quadrant layout event. */
	void OnLayoutTreeEllipseQuadrant( wxCommandEvent& event );
	
	/** Handle the popup menu tree layout line visibility event. */
	void OnTreeLayoutLineVisibility( wxCommandEvent& event );

	/** Handle the popup menu tree layout ellipse visibility event. */
	void OnTreeLayoutEllipseVisibility( wxCommandEvent& event );

	/** Handle the popup menu tree geographic axis visibility event. */
	void OnTreeGeographicAxisVisibility( wxCommandEvent& event );

	/** Handle the popup menu "Assign Geographical Axis to Tree" event. */
	void OnAssignGeoAxisToTree( wxCommandEvent& event );

	/** Reverse the order of geographic points on the geographic layout line. */
	void OnReverseGeoAxisOrder( wxCommandEvent& event );

	/** Handle the popup menu leaf node labels event. */
	void OnLeafNodeLabels( wxCommandEvent& event );

	/** Handle the popup menu significance test event. */
	void OnTreeSignificanceTest( wxCommandEvent& event );

	/** Handle the popup menu subtree significance test event. */
	void OnSubTreeSignificanceTest( wxCommandEvent& event );

	/** Handle the popup menu to extend a geographic polyline. */
	void OnExtendGeographicPolyline( wxCommandEvent& event );

	/** Layout charts evenly along a layout line. */
	void OnLayoutChartsEvenlyOnLine(wxCommandEvent& event);

	/** Project charts onto a layout line. */
	void OnLayoutChartsLineProjected(wxCommandEvent& event);

	/** Layout each quadrant of an ellipse independently. */
	void OnLayoutChartsEllipseQuadrant(wxCommandEvent& event);

	/** Layout charts evenly along a layout ellipse. */
	void OnLayoutChartsEvenlyOnEllipse(wxCommandEvent& event);

	/** Project charts onto a layout ellipse. */
	void OnLayoutChartsEllipseProjected(wxCommandEvent& event);

	/** Handle the popup menu chart set layout line event. */
	void OnChartSetLayoutLine( wxCommandEvent& event );

	/** Handle the popup menu chart set layout ellipse event. */
	void OnChartSetLayoutEllipse( wxCommandEvent& event );

	/** OpenGL render event handler. */
	void SigRender();

	/** Handle mouse move and button down events from viewport. */
	virtual void SigViewportMouse(wxMouseEvent& event);

	/** Handle mouse wheel events from viewport. */
	virtual void SigViewportMouseWheel(wxMouseEvent& event);

	/** Handle keyboard events from viewport. */
	void SigViewportKeyboard( wxKeyEvent& event );

	/** Handle node update events for trees. */
	void SigNodeUpdate(NodeInfo nodeInfo);

	/**
	 * @brief Determine if mouse is over an interactive element when mouse left down event occurs.
	 * @param mousePt Position of mouse.
	 */
	bool MouseLeftDown(const Point2D& mousePt);

	/**
	 * @brief Determine if mouse is over an interactive element when mouse left up event occurs.
	 * @param mousePt Position of mouse.
	 */
	void MouseLeftUp(const Point2D& mousePt);

	/**
	 * @brief Determine if mouse is over an interactive element when mouse right down event occurs.
	 * @param mousePt Position of mouse.
	 */
	bool MouseRightDown(const Point2D& mousePt);

	/**
	 * @brief Determine if mouse is being dragged while connected to an interactive element.
	 * @param mousePt Position of mouse.
	 */
	void MouseDragging(const Point2D& mousePt);

	/**
	 * @brief Determine if mouse is over an interactive element when float-clicked.
	 * @param mousePt Position of mouse.
	 */
	void MouseLeftDoubleClick(const Point2D& mousePt);

	/** Get widget controller. */
	WidgetControllerPtr GetWidgetController() {return m_widgetController;}
	
protected:
	/** Last position of mouse. */
	Point2D m_lastMousePos;

	/** Currently active terrain. */
	TerrainControllerPtr m_terrainController;

	/** Track position of main window horizontal splitter. */
	int m_horSplitterPos;

	/** Track position of main window vertical splitter. */
	int m_verSplitterPos;

	/** Unassigned layout line. */
	LayoutLinePtr m_layoutLine;

	/** Unassigned layout ellipse. */
	LayoutEllipsePtr m_layoutEllipse;

	/** Unassigned geographic axis defined by a polyline. */
	GeoAxisPolylinePtr m_geoAxisPolyline;

	/** Controller to manage OpenGL widgets. */
	WidgetControllerPtr m_widgetController;
	
	/** Currently active study layer. */
	StudyLayerPtr m_studyLayer;
	
	/** Currently active map layer. */
	MapLayerPtr m_mapLayer;

	/** Currently active location set layer. */
	LocationSetLayerPtr m_locationSetLayer;

	/** Currently active location layer. */
	LocationLayerPtr m_locationLayer;

	/** Currently active sequence layer. */
	SequenceLayerPtr m_sequenceLayer;

	/** Currently active tree layer. */
	TreeLayerPtr m_treeLayer;
};

}

#endif 
