//=======================================================================
// Author: Donovan Parks
//
// Copyright 2009 Donovan Parks
//
// This file is part of GenGIS.
//
// GenGIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// GenGIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GenGIS.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#include "../core/Precompiled.hpp"

#include "../gui/StudyPropertiesDlg.hpp"
#include "../gui/ProgressDlg.hpp"

#include "../utils/StringTools.hpp"

#include "../core/App.hpp"
#include "../core/Viewport.hpp"
#include "../core/StudyController.hpp"
#include "../core/StudyLayer.hpp"
#include "../core/TerrainController.hpp"
#include "../core/TerrainView.hpp"

using namespace GenGIS;

StudyPropertiesDlg::StudyPropertiesDlg(wxWindow* parent, StudyLayerPtr studyLayer)
	: StudyPropertiesLayout(parent), m_studyLayer(studyLayer)
{
	Init();
	Fit();
}

StudyPropertiesDlg::~StudyPropertiesDlg()
{

}

void StudyPropertiesDlg::Init()
{
	// set state of controls on General page
	m_txtLayerName->SetValue(wxString(m_studyLayer->GetName().c_str()));
	m_txtLayerDescription->SetValue(wxString(m_studyLayer->GetDescription().c_str()));
	m_txtAuthours->SetValue(wxString(m_studyLayer->GetAuthours().c_str()));

	// set state of controls on Projection page
	StudyControllerPtr studyController = m_studyLayer->GetStudyController();
	m_cboDatum->SetValue(wxString(studyController->GetDatum().c_str()));
	m_cboProjection->SetValue(wxString(studyController->GetProjection().c_str()));

	// set state of controls on Symbology page
	Colour bgColour = App::Inst().GetViewport()->GetBackgroundColour();
	m_colourBackground->SetColour(wxColour(bgColour.GetRedInt(), bgColour.GetGreenInt(), bgColour.GetBlueInt()));

	if(App::Inst().GetTerrainController())
	{
		if(App::Inst().GetTerrainController()->GetTerrainView()->GetMaxTerrainResolution() == 256)
			m_cboTerrainResolution->SetValue(_T("256"));
		else if(App::Inst().GetTerrainController()->GetTerrainView()->GetMaxTerrainResolution() == 512)
			m_cboTerrainResolution->SetValue(_T("512"));
		else if(App::Inst().GetTerrainController()->GetTerrainView()->GetMaxTerrainResolution() == 1024)
			m_cboTerrainResolution->SetValue(_T("1024"));
		else if(App::Inst().GetTerrainController()->GetTerrainView()->GetMaxTerrainResolution() == 2048)
			m_cboTerrainResolution->SetValue(_T("2048"));
		else if(App::Inst().GetTerrainController()->GetTerrainView()->GetMaxTerrainResolution() == 4096)
			m_cboTerrainResolution->SetValue(_T("4096"));
		else if(App::Inst().GetTerrainController()->GetTerrainView()->GetMaxTerrainResolution() == 8196)
			m_cboTerrainResolution->SetValue(_T("8196"));
	}
	else
	{
		m_cboTerrainResolution->SetValue(_T("2048"));
	}

	// set state of controls on Metadata page
	m_txtLayerSource->SetValue(wxString(m_studyLayer->GetPath().c_str()) + _T("\\") + wxString(m_studyLayer->GetFilename().c_str()));
}

void StudyPropertiesDlg::Apply()
{
	// set properties based on state of controls in General page
	m_studyLayer->SetName(m_txtLayerName->GetValue().c_str());
	m_studyLayer->SetDescription(m_txtLayerDescription->GetValue().c_str());
	m_studyLayer->SetAuthours(m_txtAuthours->GetValue().c_str());

	// set properties based on state of controls in Projection page
	StudyControllerPtr studyController = m_studyLayer->GetStudyController();
	studyController->SetDatum(m_cboDatum->GetValue().wc_str());
	studyController->SetProjection(m_cboProjection->GetValue().wc_str());

	// set properties based on state of controls in Symbology page
	App::Inst().GetViewport()->SetBackgroundColour(Colour(m_colourBackground->GetColour()));

	if(m_cboTerrainResolution->GetValue() == _T("256"))
		App::Inst().GetTerrainController()->GetTerrainView()->SetMaxTerrainResolution(256);
	else if(m_cboTerrainResolution->GetValue() == _T("512"))
		App::Inst().GetTerrainController()->GetTerrainView()->SetMaxTerrainResolution(512);
	else if(m_cboTerrainResolution->GetValue() == _T("1024"))
		App::Inst().GetTerrainController()->GetTerrainView()->SetMaxTerrainResolution(1024);
	else if(m_cboTerrainResolution->GetValue() == _T("2048"))
		App::Inst().GetTerrainController()->GetTerrainView()->SetMaxTerrainResolution(2048);
	else if(m_cboTerrainResolution->GetValue() == _T("4096"))
		App::Inst().GetTerrainController()->GetTerrainView()->SetMaxTerrainResolution(4096);
	else if(m_cboTerrainResolution->GetValue() == _T("8196"))
		App::Inst().GetTerrainController()->GetTerrainView()->SetMaxTerrainResolution(8196);

	App::Inst().GetViewport()->Refresh(false);
	App::Inst().GetLayerTreeController()->SetName(m_studyLayer->GetId(), m_studyLayer->GetName());
}

