import wx
from wx.py import shell, version

class PythonInterpreter(wx.Panel):
  """
  This class defines a wx.Panel that holds a wx.py.shell.Shell (which is a Python console).
  The wx.Panel is eventually added to GenGIS on the C++ wxWidgets side.
  """
  def __init__(self, parent):
    wx.Panel.__init__(self, parent, -1, style=wx.SUNKEN_BORDER)            # Call parent's __init__ function
    pyInterpreter = shell.Shell(self, -1)                                  # This is the actual interactive Python console.
    sizer = wx.BoxSizer(wx.VERTICAL)
    sizer.Add(pyInterpreter, 1, wx.EXPAND|wx.BOTTOM|wx.LEFT|wx.RIGHT, 10)  # Add the shell to the sizer.
    self.SetSizer(sizer)

def plotLocation(self,x,title=None):
  """
  Generates a plot for this location object. The plot is saved as a png image in the GenGIS root directory. R, RPy, and numpy are needed for this method.
  NOTE: Eventually, the plot will be automatically loaded into the program, but this is not yet implemented.
  
  `x` is the plot's x-axis: a string of any key in the location object's sequences' data. The y-axis will be a count of all sequences' values for that key.
  
  `title` is any string, and is used for both the title of the plot and the name of the image file generated.
  """
  # If not already imported, import rpy.r:
  try:
    import rpy2.robjects as robjects
    r = robjects.r
  except:
    print 'Module rpy2 not found. Cannot continue.'
    return
  # If not already imported, import numpy:
  #try: import numpy
  #except:
  #  print 'Module numpy not found. Cannot continue.'
  #  return
  # If the title of the plot is not given, use the name of the file.
  if title is None or type(title) is not str: title = str(self.name) + ": " + str(x)
  plotData = {}
  # Get the data from the location:
  for i in xrange(0,len(self.sequences)):
    if x in self.sequences[i].data:
      if self.sequences[i].data[x] in plotData: plotData[self.sequences[i].data[x]] += 1
      else:                                     plotData[self.sequences[i].data[x]] = 1
  # We might need to adjust the margins and labels:
  defaultXLabel = 4
  defaultXMargin = 6
  xLabelMax = len(str(max(plotData.keys(),key=lambda x: len(str(x)))))
  if xLabelMax > 5:
    defaultXLabel = 5+((xLabelMax-5)/2)-(.005*xLabelMax*xLabelMax)   # This formula sets the position of the label and margin based on the # of characters in the longest word on the axis (`xLabelMax`).
    defaultXMargin = defaultXLabel+2                                 # This is the position of the margin.
  # And repeat for y:
  defaultYLabel = 4
  defaultYMargin = 6
  yLabelMax = len(str(max(plotData.values(),key=lambda x: len(str(x)))))
  if yLabelMax > 5:
    defaultYLabel = 5+((yLabelMax-5)/2)-(.005*yLabelMax*yLabelMax)   # This formula sets the position of the label and margin based on the # of characters in the longest word on the axis (`xLabelMax`).
    defaultYMargin = defaultYLabel+2                                 # This is the position of the margin.
  # Do the RPy plotting:
  #####
  # ~mikep. png plotting must be possible with rpy2, but I have nop idea hopw to do it.
  #r.png(title+'.png')
  #####
  #####
  #r.par(mar=(defaultXMargin,defaultYMargin,5,0))
  #####
  if xLabelMax > 5: r.barplot(**{"height":robjects.IntVector(plotData.values()), "names.arg":plotData.keys(), "main":title, "col":'blue', "las":2})
  else:             r.barplot(**{"height":robjects.IntVector(plotData.values()), "names.arg":plotData.keys(), "main":title, "col":'blue'})
  #####
  #r.mtext(x, side=1, line=defaultXLabel)
  #r.mtext("Count", side=2, line=defaultYLabel)
  #####
  #r.dev_off()
  #####


def plotEachSample(image,x,y=None,plotType="automatic",title=None,load=True,delete=True):
  """
  Generates a plot with R (via RPy) for each sample.
  
  `x` and `y` are the data that will be plotted, as a string. They must be a valid taxa column name.
  
  `image` is the path to the image file where the plot is saved. The unique id of each sample is appended to `image` to ensure each plot has a different name.
  The plot's title is `title`, if `title` is None the title is the file name.
  
  If `plotType` is "scatter" a scatter plot will be produced. "bar" produces a bar plot. "automatic" lets the API use the plot type it thinks is best.
  
  If `load` is True, the file is loaded into gviz.
  If `delete` is True, the file is deleted from the hard drive (after being loaded if `load` is True).
  
  Plots loaded into gviz must be saved to the hard drive first, then loaded, and then deleted. This is because of compatibility issues between R and wxPython.
  """
  # If not already imported, import rpy.r:
  try: from rpy import r
  except:
    print 'Module rpy not found. Cannot continue.'
    return
  # If not already imported, import numpy:
  try: import numpy
  except:
    print 'Module numpy not found. Cannot continue.'
    return
  ##########
  ##########    update to use GenGISAPIv1:
  ##########
  ##########
  ##########for eachLocation in samples(): plotSample(image+"_"+eachSample.id,x,eachSample.id,y=y,plotType=plotType,title=title,load=load,delete=delete)
  ##########
  ##########
  ##########
  ##########

# def plotAggregateSamples(image,x,y=None,plotType="automatic",title=None,load=True,delete=True):
#   '''
#   
#   '''
#   # If not already imported, import rpy.r:
#   try: from rpy import r
#   except:
#     print 'Module rpy not found. Cannot continue.'
#     return
#   # If not already imported, import numpy:
#   try: import numpy
#   except:
#     print 'Module numpy not found. Cannot continue.'
#     return
#   # If x is a list, no processing needs to be done to it, otherwise, look it up based on the given string
#   xData = []
#   if type(x) is str:
#       for eachSample in samples():
#           # Do they want a column in the sample?
#           try: xData.append(eachSample.properties[x])
#           # Or in the taxa?
#           except:
#               for eachTaxon in eachSample.taxa: xData.append(eachTaxon.properties[x])
#       xLabel = str(x)
#       if xData is []:
#           error('Cannot plot graph; no data found.')
#           return
#   else:
#       error("Cannot plot graph; unknown value for x.")
#       return
#   yData = []
#   if type(y) is str:
#       for eachSample in samples():
#           # Do they want a column in the sample?
#           try: yData.append(eachSample.properties[y])
#           # Or in the taxa?
#           except:
#               for eachTaxon in eachSample.taxa: yData.append(eachTaxon.properties[y])
#       yLabel = str(y)
#       if yData is []:
#           error('Cannot plot graph; no data found.')
#           return
#   elif y is not None:
#       error("Cannot plot graph; unknown value for y.")
#       return
#   # If the title of the plot is not given, use the name of the file.
#   if title is None: title = image
#   # We might need to adjust the margins and labels:
#   defaultXLabel = 4
#   defaultXMargin = 6
#   xLabelMax = 0
#   for i in xData:
#       if len(str(i)) > xLabelMax: xLabelMax = len(i)
#   if xLabelMax > 5:
#       defaultXLabel = 5+((xLabelMax-5)/2)-(.005*xLabelMax*xLabelMax)   # This formula sets the position of the label and margin based on the # of characters in the longest word on the axis (`xLabelMax`).
#       defaultXMargin = defaultXLabel+2                                 # This is the position of the margin.
#   # And repeat for y:
#   defaultYLabel = 4
#   defaultYMargin = 6
#   yLabelMax = 0
#   for i in yData:
#       if len(str(i)) > yLabelMax: yLabelMax = len(i)
#   if yLabelMax > 5:
#       defaultYLabel = 5+((yLabelMax-5)/2)-(.005*yLabelMax*yLabelMax)   # This formula sets the position of the label and margin based on the # of characters in the longest word on the axis (`xLabelMax`).
#       defaultYMargin = defaultYLabel+2                                 # This is the position of the margin.
#   # Get the count
#   if y is None:
#       xTempData = []
#       [xTempData.append(i) for i in xData if not xTempData.count(i)]
#       yData = []
#       for i in xTempData: yData.append(xData.count(i))
#       xData = xTempData
#       # Do the RPy plotting:
#       r.png(image)
#       r.par(mar=(defaultXMargin,defaultYMargin,5,0))
#       if xLabelMax > 5: r.barplot(yData, names_arg=xData, main=title, col='blue', las=2)
#       else:             r.barplot(yData, names_arg=xData, main=title, col='blue')
#       r.mtext(xLabel, side=1, line=defaultXLabel)
#       r.mtext("Count", side=2, line=defaultYLabel)
#       r.dev_off()
#   else:
#       # yData is a list at this point
#       r.png(image)
#       r.par(mar=(defaultXMargin,defaultYMargin,5,1))
#       if xLabelMax > 5: r.plot(xData, yData, main=title, xlab="", ylab="", col='blue', las=2)   # Turn the labels sideways if they are long
#       else:             r.plot(xData, yData, main=title, xlab="", ylab="", col='blue')
#       r.mtext(xLabel, side=1, line=defaultXLabel)
#       r.mtext(yLabel, side=2, line=defaultYLabel)
#       r.dev_off()
#   if load: openImage(image)               # If requested, load the image
#   if delete: os.remove(image)             # If requested, delete the image
# 



# def plotBySample(image,x,plotType="automatic",title=None,load=True,delete=True):
#   """
#   automatic | bar | stacked
#   
#   note: if x is None, then all categories should be plotted together... FIX
#   """
#   if 'r' not in dir() or 'numpy' not in dir(): error("Cannot plot graph; the Python modules 'rpy' and 'numpy' were not found.\nPlease install these modules first.\n")
#   global win
#   # verify valid `plotType`
#   if plotType not in ('automatic', 'bar', 'stacked'):
#       error("Invalid plotType: "+str(plotType)+". Using plotType='automatic' instead.")
#       plotType = 'automatic'
#   if plotType == 'automatic' or plotType == 'bar': yData = []
#   elif plotType == 'stacked':                      yData = {}
#   xDataLabels = []
#   if type(x) is str:
#       # parse x:
#       xList = x.split('[')
#       xClauses = ','
#       xColumn = xList[0]
#       if xColumn not in r('names(taxa)'):
#           error("Cannot plot graph; unknown value for x.")
#           return
#       if len(xList) > 2:
#           error("Invalid syntax in x.")
#           return
#       if len(xList) == 2:
#           xList = re.findall(r'[^\|&]+[\|&]?',xList[1][:-1])
#           xList = ['taxa$'+str(i) for i in xList]
#           xClauses = ''.join(xList)
#           xClauses = '&'+xClauses
#       for eachSample in samples():
#           xData = []
#           xDataLabels.append("Site "+str(eachSample.id))
#           xData = r('taxa[taxa$site=="'+str(eachSample.id)+'"'+xClauses+']$'+str(xColumn))
#           if plotType == 'automatic' or plotType == 'bar':
#               yData.append(len(xData))
#           elif plotType == 'stacked':
#               xUniqueData = []
#               xSeenData = set()
#               for i in xData:
#                   if i in xSeenData: continue
#                   xUniqueData.append(i)
#                   xSeenData.add(i)
#               for i in yData.keys():
#                   if i in xUniqueData:
#                       yData[i].append(xData.count(i))
#                       xUniqueData.remove(i)
#                   else:
#                       yData[i].append(0)
#               for i in xUniqueData:
#                   yDataKeys = yData.keys()
#                   if yDataKeys == []: yTempData = []
#                   else:               yTempData = yData[yDataKeys[0]]
#                   yNewData = []
#                   if len(yTempData)-1 > 0:
#                       for j in xrange(len(yTempData)-1): yNewData.append(0)
#                   yNewData.append(xData.count(i))
#                   yData[i] = yNewData
#       xLabel = str(x)
#   else:
#       error("Cannot plot graph; unknown value for x.")
#       return
#   # If the title of the plot is not given, use the name of the file.
#   if title is None: title = image
#   # We might need to adjust the margins and labels:
#   defaultXLabel = 4
#   defaultXMargin = 6
#   xLabelMax = 0
#   for i in xDataLabels:
#       if len(str(i)) > xLabelMax: xLabelMax = len(i)
#   if xLabelMax > 5:
#       defaultXLabel = 5+((xLabelMax-5)/2)-(.005*xLabelMax*xLabelMax)   # This formula sets the position of the label and margin based on the # of characters in the longest word on the axis (`xLabelMax`).
#       defaultXMargin = defaultXLabel+2                                 # This is the position of the margin.
#   # Do the RPy plotting:
#   r.png(image)
#   r.par(mar=(defaultXMargin,6,5,0))
#   if plotType == 'automatic' or plotType == 'bar':
#       if xLabelMax > 5: r.barplot(yData, names_arg=xDataLabels, main=title, col='blue', las=2)
#       else:             r.barplot(yData, names_arg=xDataLabels, main=title, col='blue')
#   if plotType == 'stacked':
#       yDataKeys = yData.keys()
#       yTempData = []
#       for i in yData.keys(): yTempData.append(yData[i])
#       yData = numpy.array(yTempData)
#       yData = yData.transpose()
#       if xLabelMax > 5: r.barplot(r.t(yData), names_arg=xDataLabels, main=title, col=r.rainbow(len(yDataKeys)), las=2)
#       else:             r.barplot(r.t(yData), names_arg=xDataLabels, main=title, col=r.rainbow(len(yDataKeys)))
#       r.legend("topright", yDataKeys, fill=r.rainbow(len(yDataKeys)))
#   r.mtext(xLabel, side=1, line=defaultXLabel)
#   r.mtext("Count", side=2, line=4)
#   r.dev_off()
#   if load: openImage(image)               # If requested, load the image
#   if delete: os.remove(image)             # If requested, delete the image