//=======================================================================
// Author: Donovan Parks
//
// Copyright 2009 Donovan Parks
//
// This file is part of GenGIS.
//
// GenGIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// GenGIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GenGIS.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#ifndef _GENGIS_GRAPH_VIZ_IO
#define _GENGIS_GRAPH_VIZ_IO

#include "../core/Precompiled.hpp"

#include "../utils/Tree.hpp"
#include "../core/NodeGeoTree.hpp"

namespace GenGIS
{

/**
 * @brief Read/write tree data in GraphViz dot format.
 */
class GraphVizIO
{
public:		
		/**
	 * @brief Write out tree in GraphViz dot format.
	 * @param path Path to write file to.
	 * @param filename Filename of output file.
	 * @param tree Tree to write out.
	 * @param map Map tree resides on.
	 * @param bFixLeafNodes Flag indicating if leaf nodes should be considered to have fixed coordinates.
	 */
	static bool Write(const std::wstring& path, const std::wstring& filename, Tree<NodeGeoTree>::Ptr tree, TerrainModelPtr map, bool bFixLeafNodes);
};

} 

#endif

