//=======================================================================
// Author: Donovan Parks
//
// Copyright 2009 Donovan Parks
//
// This file is part of GenGIS.
//
// GenGIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// GenGIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GenGIS.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#ifndef _GENGIS_COMPASSVIEW2D_
#define _GENGIS_COMPASSVIEW2D_

#include "../core/Precompiled.hpp"

#include "../widgets/ViewWidget.hpp"

namespace GenGIS
{

class CompassView2D : public ViewWidget 
{
public:        
  CompassView2D() {}
  CompassView2D( float theta );
  
  void setAngle( float theta );
  
  float getAngle() { return m_theta; }
  
  void setPivotPoint( float x, float y ) 
	{
      m_pivot.x = x;
      m_pivot.y = y;
  }

  const Point2D& getPivotPoint() const { return m_pivot; }
  const Point2D& getRadii() const { return m_radii; }
  
  void setRadii( float x, float y ) 
	{
      m_radii.x = x;
      m_radii.y = y;
  }
    
  void Render();

	/** 
	 * @brief Process mouse left button down events. 
	 * @param mousePt Screen position of mouse.
	 * @return True if view is processing the event, else false.
	 */
	bool MouseLeftDown(const Point2D& mousePt);

	/** 
	 * @brief Process mouse left button float click events. 
	 * @param mousePt Screen position of mouse.
	 * @return True if view is processing the event, else false.
	 */
	void MouseLeftDblClick(const Point2D& mousePt) { MouseLeftDown(mousePt); }

	/** 
	 * @brief Process mouse dragging events. 
	 * @param mousePt Screen position of mouse.
	 */
	void MouseDragging(const Point2D& mousePt) { MouseLeftDown(mousePt); return; }

private:
	float angleOnCompass(const Point2D& mousePt);
        
private:
  float m_theta;
  Point2D m_pivot;
  Point2D m_radii;		
  void drawTickMarks();
  void drawNorthNeedle();        
};

}

#endif

