//=======================================================================
// Author: Donovan Parks
//
// Copyright 2009 Donovan Parks
//
// This file is part of GenGIS.
//
// GenGIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// GenGIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GenGIS.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#ifndef _GENGIS_MINIMAPVIEW2D_
#define _GENGIS_MINIMAPVIEW2D_

#include "../core/Precompiled.hpp"

#include "../widgets/ViewWidget.hpp"

namespace GenGIS
{

class MiniMapView2D : public ViewWidget 
{
public:
  MiniMapView2D();
	~MiniMapView2D() {}

  void Render();
        
	void SetTerrainController(TerrainControllerPtr terrainController) { m_terrainController = terrainController; }
	void SetLayerTree(LayerTreePtr layerTree) { m_layerTree = layerTree; }

  void SetOrigin( float x, float y )	{ m_origin.x = x; m_origin.y = y; }
  const Point2D& GetOrigin() const { return m_origin; }

	Point3D GeoCoordToMiniMapCoord( GeoCoord& geoCoord );
	GeoCoord MiniMapCoordToGeoCoord( Point3D& miniMapCoord );
  
	void MiniMapCoordToWorldCoord( Point3D& miniMapCoord, Point3D& worldCoord );  
	Point3D WorldCoordToMiniMapCoord( Point3D& worldCoord );

	/** 
	 * @brief Process mouse left button down events. 
	 * @param mousePt Screen position of mouse.
	 * @return True if view is processing the event, else false.
	 */
	virtual bool MouseLeftDown(const Point2D& mousePt);

	/** 
	 * @brief Process mouse left button float click events. 
	 * @param mousePt Screen position of mouse.
	 * @return True if view is processing the event, else false.
	 */
	void MouseLeftDblClick(const Point2D& mousePt) { MouseLeftDown(mousePt); }

	/** 
	 * @brief Process mouse dragging events. 
	 * @param mousePt Screen position of mouse.
	 */
	void MouseDragging(const Point2D& mousePt) { MouseLeftDown(mousePt); return; }
  
private:
	bool IsClicked(const Point2D& mousePt);

	void DrawTerrain();

private:  
  Point2D m_origin;

	TerrainControllerPtr m_terrainController;
	
	LayerTreePtr m_layerTree;
};

}

#endif

