//=======================================================================
// Author: Donovan Parks
//
// Copyright 2009 Donovan Parks
//
// This file is part of GenGIS.
//
// GenGIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// GenGIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GenGIS.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#ifndef _GENGIS_MINIMAPVIEW2DCONTROLLER_
#define _GENGIS_MINIMAPVIEW2DCONTROLLER_

#include "../widgets/MiniMapView2D.hpp"

#include "../core/ViewController.hpp"
#include "../core/Camera.hpp"
#include "../core/LayerTree.hpp"

namespace GenGIS
{

class MiniMapView2DController : public ViewController 
{
public:
	/** Convenient syntax for creating smart pointers to this class */
	typedef boost::shared_ptr<MiniMapView2DController> Ptr;

public:	  
	MiniMapView2DController(Camera::Ptr camera);
  
	ViewWidget::Ptr GetView() { return getMiniMapView2D(); }
	MiniMapView2D::Ptr getMiniMapView2D();

	bool IsClicked(int mouseX, int mouseY);
          
	void SetTerrainController(TerrainController::Ptr terrainController) { m_miniMapView->SetTerrainController(terrainController); }
	void SetLayerTree(LayerTree::Ptr layerTree) { m_miniMapView->SetLayerTree(layerTree); }
  
	Point3D mouseLeftDown( double u, double v );
    
private:  
	Camera::Ptr m_camera;
  
	MiniMapView2D::Ptr m_miniMapView;
};

}

#endif

