//=======================================================================
// Author: Donovan Parks
//
// Copyright 2009 Donovan Parks
//
// This file is part of GenGIS.
//
// GenGIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// GenGIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GenGIS.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#ifndef _GENGIS_NAVVIEW2D_
#define _GENGIS_NAVVIEW2D_

#include "../core/Precompiled.hpp"

#include "../widgets/ViewWidget.hpp"

namespace GenGIS
{

class NavView2D : public ViewWidget 
{
public:
  NavView2D();
  
  const Point2D& getZoomInOrigin() const { return m_zoomInOrigin; }
  void setZoomInOrigin( float x, float y ) {
      m_zoomInOrigin.x = x;
      m_zoomInOrigin.y = y;
  }
  
  const Point2D& getZoomOutOrigin() const { return m_zoomOutOrigin; }
  void setZoomOutOrigin( float x, float y ) {
      m_zoomOutOrigin.x = x;
      m_zoomOutOrigin.y = y;
  }
  
  const Point2D& getZoomInRadii() const { return m_zoomInRadii; }
  void setZoomInRadii( float x, float y ) {
      m_zoomInRadii.x = x;
      m_zoomInRadii.y = y;
  }
  
  const Point2D& getZoomOutRadii() const { return m_zoomOutRadii; }
  void setZoomOutRadii( float x, float y ) {
      m_zoomOutRadii.x = x;
      m_zoomOutRadii.y = y;
  }

	const Point2D& getLeftArrowOrigin() const { return m_leftArrowOrigin; }
  void setLeftArrowOrigin( float x, float y ) {
      m_leftArrowOrigin.x = x;
      m_leftArrowOrigin.y = y;
  }
  
  const Point2D& getRightArrowOrigin() const { return m_rightArrowOrigin; }
  void setRightArrowOrigin( float x, float y ) {
      m_rightArrowOrigin.x = x;
      m_rightArrowOrigin.y = y;
  }
  
  const Point2D& getLeftArrowRadii() const { return m_leftArrowRadii; }
  void setLeftArrowRadii( float x, float y ) {
      m_leftArrowRadii.x = x;
      m_leftArrowRadii.y = y;
  }
  
  const Point2D& getRightArrowRadii() const { return m_rightArrowRadii; }
  void setRightArrowRadii( float x, float y ) {
      m_rightArrowRadii.x = x;
      m_rightArrowRadii.y = y;
  }

	const Point2D& getUpArrowOrigin() const { return m_upArrowOrigin; }
  void setUpArrowOrigin( float x, float y ) {
      m_upArrowOrigin.x = x;
      m_upArrowOrigin.y = y;
  }
  
  const Point2D& getDownArrowOrigin() const { return m_downArrowOrigin; }
  void setDownArrowOrigin( float x, float y ) {
      m_downArrowOrigin.x = x;
      m_downArrowOrigin.y = y;
  }
  
  const Point2D& getUpArrowRadii() const { return m_upArrowRadii; }
  void setUpArrowRadii( float x, float y ) {
      m_upArrowRadii.x = x;
      m_upArrowRadii.y = y;
  }
  
  const Point2D& getDownArrowRadii() const { return m_downArrowRadii; }
  void setDownArrowRadii( float x, float y ) {
      m_downArrowRadii.x = x;
      m_downArrowRadii.y = y;
  }

	/** 
	 * @brief Process mouse left button down events. 
	 * @param mousePt Screen position of mouse.
	 * @return True if view is processing the event, else false.
	 */
	virtual bool MouseLeftDown(const Point2D& mousePt);

	/** 
	 * @brief Process mouse left button float click events. 
	 * @param mousePt Screen position of mouse.
	 * @return True if view is processing the event, else false.
	 */
	void MouseLeftDblClick(const Point2D& mousePt) { MouseLeftDown(mousePt); }

protected:
  bool IsOnZoomIn(const Point2D& mousePt);
  bool IsOnZoomOut(const Point2D& mousePt);
  bool IsOnLeftArrow(const Point2D& mousePt);
	bool IsOnRightArrow(const Point2D& mousePt);
	bool IsOnUpArrow(const Point2D& mousePt);
	bool IsOnDownArrow(const Point2D& mousePt);

	bool IsInEllipse(const Point2D& mousePt, const Point2D& origin, const Point2D& radii);

protected:
  Point2D m_zoomInOrigin;
  Point2D m_zoomOutOrigin;
  Point2D m_zoomInRadii;
  Point2D m_zoomOutRadii;

  Point2D m_leftArrowOrigin;
  Point2D m_rightArrowOrigin;
  Point2D m_leftArrowRadii;
  Point2D m_rightArrowRadii;

	Point2D m_upArrowOrigin;
  Point2D m_downArrowOrigin;
  Point2D m_upArrowRadii;
  Point2D m_downArrowRadii;
};

}

#endif

