//=======================================================================
// Author: Donovan Parks
//
// Copyright 2009 Donovan Parks
//
// This file is part of GenGIS.
//
// GenGIS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// GenGIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with GenGIS.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#ifndef _GENGIS_VIEW2D_
#define _GENGIS_VIEW2D_

#include "../core/Precompiled.hpp"

#include "../core/View.hpp"

namespace GenGIS
{

/**
 * @brief Abstract base class for 2D (orthographic) widgets.
 */
class ViewWidget : public View 
{
public:
  ViewWidget() { SetScale( 1, 1, 1 ); m_textured = false; }

  virtual void Render();

  virtual Point3D GetScreenCoordinates() { return m_screenCoordinates; }
  virtual void SetScreenCoords( const Point3D& screenC  );

	virtual const Point3D& GetScale() const {	return m_scale; }

  virtual void SetScale( float sx, float sy, float sz = 1 ) 
	{
    m_scale.x = sx;
    m_scale.y = sy;
    m_scale.z = sz;
  }            
  
  virtual bool IsTextured() { return m_textured; }
  
	virtual void SetTexture( TexturePtr texture );

	virtual Box2D GetBoundingBox();

protected:
	Point3D m_scale;
  bool m_textured;
	TexturePtr m_texture;
  Point3D m_screenCoordinates;
};

}

#endif

